package de.moritzruth.theaterdsl.value

@JvmInline
value class Angle(val degrees: Double) : Comparable<Angle> {
    override fun toString(): String = "${degrees}°"

    override fun compareTo(other: Angle): Int = degrees.compareTo(other.degrees)

    fun inSingleRotation(): Angle = Angle(
        if (degrees == 360.0 || degrees == -360.0) degrees
        else degrees.mod(360f)
    )

    class Range(
        override val start: Angle,
        override val endInclusive: Angle
    ): ClosedFloatingPointRange<Angle> {
        override fun lessThanOrEquals(a: Angle, b: Angle): Boolean = a.degrees <= b.degrees

        fun asDoubleRange(): ClosedFloatingPointRange<Double> = start.degrees..endInclusive.degrees

        companion object {
            val FULL_ROTATION: Range = Range(Angle(0.0), Angle(360.0))
        }
    }
}

inline val Int.degrees: Angle
    get() = Angle(this.toDouble())

inline val Double.degrees: Angle
    get() = Angle(this)

fun Double.asAngle(): Angle = Angle(this)
