package de.moritzruth.theaterdsl.value

@JvmInline
value class Frequency(val hertz: Double) : Comparable<Frequency> {
    companion object {
        val ZERO: Frequency = Frequency(0.0)
        val INFINITY: Frequency = Frequency(Double.POSITIVE_INFINITY)
    }

    init {
        require(hertz >= 0) { "must be at least 0" }
    }

    override fun toString(): String = "${hertz}hz"

    override fun compareTo(other: Frequency): Int = hertz.compareTo(other.hertz)

    class Range(
        override val start: Frequency,
        override val endInclusive: Frequency
    ) : ClosedFloatingPointRange<Frequency> {
        override fun lessThanOrEquals(a: Frequency, b: Frequency): Boolean = a.hertz <= b.hertz

        fun asDoubleRange(): ClosedFloatingPointRange<Double> = start.hertz..endInclusive.hertz

        override fun toString(): String = "$start..$endInclusive"
    }
}
